class longstr(str):  # noqa: N801
    """Long string functions mostly as a type annotation for the web."""

    pass


class fileurl(str):  # noqa: N801
    """Type alias that, if used in a package method argument, will cause a file upload widget to appear."""

    pass
